<?php
/**
 * This is the entrypoint for the locrMAPS for WooCommerce WordPress plugin.
 *
 * @package locrMAPS for WooCommerce
 */

declare(strict_types=1);

namespace Locr\WordPress\Plugin\LocrMapsForWooCommerce\Admin;

use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\ValueObjects\{IconsQuerySelector, Options, Size, SizesQuerySelector};
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\DDD\Repositories\Styles as StylesRepository;
use Locr\WordPress\Plugin\LocrMapsForWooCommerce\Plugin;

if ( ! defined( 'ABSPATH' ) ) {
	exit; // Exit if accessed directly.
}

/**
 * This class is responsible for the settings page.
 */
final class SettingsPage {
	/**
	 * This function initializes the settings page.
	 */
	public static function init(): void {
		add_action( 'locrmaps_after_add_menu', array( self::class, 'add_submenu_page' ) );
		add_action( 'admin_init', array( self::class, 'register_settings' ) );
	}

	/**
	 * This function adds the submenu page.
	 */
	public static function add_submenu_page(): void {
		add_submenu_page(
			parent_slug: 'locrmaps',
			page_title: __( 'WooCommerce', 'locrmaps-for-woocommerce-textdomain' ),
			menu_title: __( 'WooCommerce', 'locrmaps-for-woocommerce-textdomain' ),
			capability: 'manage_options',
			menu_slug: Plugin::SLUG,
			callback: array( self::class, 'render_submenu_page' ),
			position: 4
		);
	}

	/**
	 * This function registers the settings.
	 */
	public static function register_settings(): void {
		register_setting( 'locrmaps_for_woocommerce_options_group', 'locrmaps_for_woocommerce_options' );

		add_settings_section(
			id: Plugin::SLUG . '_section',
			title: __( 'Basic settings', 'locrmaps-for-woocommerce-textdomain' ),
			callback: array( self::class, 'basic_settings_section_callback' ),
			page: Plugin::SLUG
		);

		add_settings_field(
			id: SizesQuerySelector::OPTION_KEY,
			title: __( 'Sizes query selector', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'sizes_query_selector_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => SizesQuerySelector::OPTION_KEY )
		);

		add_settings_field(
			id: Plugin::SLUG . '_icons',
			title: __( 'Icons', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'icons_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_icons' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_icons_shortcode',
			title: __( 'Icons Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_icons_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_icons_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_styles',
			title: __( 'Styles', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'styles_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_styles' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_styles_shortcode',
			title: __( 'Styles Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_styles_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_styles_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates',
			title: __( 'OverlayTemplates', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'overlay_templates_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates_shortcode',
			title: __( 'OverlayTemplates Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_overlay_templates_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates_base_colors',
			title: __( 'OverlayTemplates Base Colors', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'overlay_templates_base_colors_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates_base_colors' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates_base_colors_shortcode',
			title: __( 'OverlayTemplates Base Colors Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_overlay_templates_base_colors_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates_base_colors_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates_text_colors',
			title: __( 'OverlayTemplates Text Colors', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'overlay_templates_text_colors_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates_text_colors' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_overlay_templates_text_colors_shortcode',
			title: __( 'OverlayTemplates Text Colors Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_overlay_templates_text_colors_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_overlay_templates_text_colors_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_text_overlay_shortcode',
			title: __( 'TextOverlay Shortcode', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'shortcode_for_text_overlay_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_text_overlay_shortcode' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_selection_layout',
			title: __( 'Selection Layout', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'selection_layout_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_selection_layout' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_selection_layout_label_enabled',
			title: __( 'Selection Layout Label Enabled', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'selection_layout_label_enabled_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_selection_layout_label_enabled' )
		);

		add_settings_field(
			id: Plugin::SLUG . '_preview_in_wc_order_disabled',
			title: __( 'Preview in WooCommerce Order Disabled', 'locrmaps-for-woocommerce-textdomain' ) . ':',
			callback: array( self::class, 'preview_in_wc_order_disabled_callback' ),
			page: Plugin::SLUG,
			section: Plugin::SLUG . '_section',
			args: array( 'label_for' => Plugin::SLUG . '_preview_in_wc_order_disabled' )
		);
	}

	/**
	 * This function renders the section.
	 */
	public static function basic_settings_section_callback(): void {
		// do nothing.
	}

	/**
	 * This function renders the icons field.
	 */
	public static function icons_query_selector_callback(): void {
		$options = new Options( only_option: IconsQuerySelector::OPTION_KEY );

		$locrmaps_for_woocommerce_icons_query_selector_placeholder = 'select#icon';
		print '<input type="text" class="regular-text" name="locrmaps_for_woocommerce_options[' . esc_attr( IconsQuerySelector::OPTION_KEY ) . ']" id="' . esc_attr( IconsQuerySelector::OPTION_KEY ) . '" placeholder="' . esc_attr( $locrmaps_for_woocommerce_icons_query_selector_placeholder ) . '" value="' . esc_attr( (string) $options->icons_query_selector ) . '"/>';
	}

	/**
	 * This function renders the icons field.
	 */
	public static function icons_callback(): void {
		$locrmaps_for_woocommerce_icons = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_icons'] ) && is_string( $options['locrmaps_for_woocommerce_icons'] ) ) {
			$locrmaps_for_woocommerce_icons = trim( $options['locrmaps_for_woocommerce_icons'] );
		}

		$locrmaps_for_woocommerce_icons_placeholder = 'pin | circle | heart';
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_icons]" id="locrmaps_for_woocommerce_icons" rows="5" placeholder="' . esc_attr( $locrmaps_for_woocommerce_icons_placeholder ) . '">' . esc_html( $locrmaps_for_woocommerce_icons ) . '</textarea>';
	}

	/**
	 * This function renders the overlay_templates field.
	 */
	public static function overlay_templates_callback(): void {
		$locrmaps_for_woocommerce_overlay_templates = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates'] ) ) {
			$locrmaps_for_woocommerce_overlay_templates = trim( $options['locrmaps_for_woocommerce_overlay_templates'] );
		}

		$locrmaps_for_woocommerce_overlay_templates_placeholder = 'clean | circle | heart | pin';
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_overlay_templates]" id="locrmaps_for_woocommerce_overlay_templates" rows="5" placeholder="' . esc_attr( $locrmaps_for_woocommerce_overlay_templates_placeholder ) . '">' . esc_html( $locrmaps_for_woocommerce_overlay_templates ) . '</textarea>';
	}

	/**
	 * This function renders the overlay_templates_base_color field.
	 */
	public static function overlay_templates_base_colors_callback(): void {
		$option_value = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] ) ) {
			$option_value = trim( $options['locrmaps_for_woocommerce_overlay_templates_base_colors'] );
		}

		$placeholder = 'white | black | #ff0000 | green | rgb(0, 0, 255) | [color name="locr" value="#ff7300"]';
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_overlay_templates_base_colors]" id="locrmaps_for_woocommerce_overlay_templates_base_colors" rows="5" placeholder="' . esc_attr( $placeholder ) . '">' . esc_html( $option_value ) . '</textarea>';
	}

	/**
	 * This function renders the overlay_templates_text_color field.
	 */
	public static function overlay_templates_text_colors_callback(): void {
		$option_value = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] ) && is_string( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] ) ) {
			$option_value = trim( $options['locrmaps_for_woocommerce_overlay_templates_text_colors'] );
		}

		$placeholder = 'white | black | #ff0000 | green | rgb(0, 0, 255) | [color name="locr" value="#ff7300"]';
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_overlay_templates_text_colors]" id="locrmaps_for_woocommerce_overlay_templates_text_colors" rows="5" placeholder="' . esc_attr( $placeholder ) . '">' . esc_html( $option_value ) . '</textarea>';
	}

	/**
	 * This function renders the preview_in_wc_order_disabled field.
	 */
	public static function preview_in_wc_order_disabled_callback(): void {
		$selection_layout_enabled = '';

		$options = get_option( Plugin::SLUG . '_options' );
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ] ) && is_string( $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ] ) ) {
			$selection_layout_enabled = checked( 'on', $options[ Plugin::SLUG . '_preview_in_wc_order_disabled' ], false );
		}

		print '<input type="checkbox" name="' . esc_attr( Plugin::SLUG . '_options[' . Plugin::SLUG . '_preview_in_wc_order_disabled]' ) . '" id="' . esc_attr( Plugin::SLUG . '_preview_in_wc_order_disabled' ) . '"' . esc_attr( $selection_layout_enabled ) . '/>';
	}

	/**
	 * This function renders the sizes field.
	 */
	public static function sizes_callback(): void {
		$locrmaps_for_woocommerce_sizes = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_sizes'] ) && is_string( $options['locrmaps_for_woocommerce_sizes'] ) ) {
			$locrmaps_for_woocommerce_sizes = trim( $options['locrmaps_for_woocommerce_sizes'] );
		}

		if ( ! empty( $locrmaps_for_woocommerce_sizes ) ) {
			$shortcodes = explode( '|', $locrmaps_for_woocommerce_sizes );
			foreach ( $shortcodes as $index => $shortcode ) {
				try {
					Size::build_from_shortcode( $shortcode );
				} catch ( \Exception $e ) {
					add_settings_error(
						setting: 'locrmaps_for_woocommerce_options',
						code: 'locrmaps_for_woocommerce_sizes',
						// translators: %d: is the shortcode index.
						// translators: %s: is the error message.
						message: sprintf( __( 'The shortcode with the index [%1$d] is invalid. %2$s.', 'locrmaps-for-woocommerce-textdomain' ), $index, $e->getMessage() ),
						type: 'error'
					);
				}
			}

			settings_errors( 'locrmaps_for_woocommerce_options' );
		}

		$locrmaps_for_woocommerce_sizes_placeholder = '[size width="21" height="30"] | [size width="30" height="40" name="Poster 30x40cm" unit="cm" dpi="300"]';
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_sizes]" id="locrmaps_for_woocommerce_sizes" rows="5" placeholder="' . esc_attr( $locrmaps_for_woocommerce_sizes_placeholder ) . '">' . esc_html( $locrmaps_for_woocommerce_sizes ) . '</textarea>';
	}

	/**
	 * This function renders the styles field.
	 */
	public static function styles_callback(): void {
		$locrmaps_for_woocommerce_styles = '';

		$options = get_option( 'locrmaps_for_woocommerce_options' );
		if ( is_array( $options ) && isset( $options['locrmaps_for_woocommerce_styles'] ) && is_string( $options['locrmaps_for_woocommerce_styles'] ) ) {
			$locrmaps_for_woocommerce_styles = trim( $options['locrmaps_for_woocommerce_styles'] );
		}

		$styles_repository                           = new StylesRepository();
		$styles                                      = $styles_repository->get_all();
		$styles[]                                    = '[style name="4201" display-name="default" overlay-template-base-color="black" overlay-template-text-color="white"]';
		$locrmaps_for_woocommerce_styles_placeholder = implode( ' | ', $styles );
		print '<textarea class="regular-text" name="locrmaps_for_woocommerce_options[locrmaps_for_woocommerce_styles]" id="locrmaps_for_woocommerce_styles" rows="5" placeholder="' . esc_attr( $locrmaps_for_woocommerce_styles_placeholder ) . '">' . esc_html( $locrmaps_for_woocommerce_styles ) . '</textarea>';
	}

	/**
	 * This function renders the seletion layout field.
	 */
	public static function selection_layout_callback(): void {
		$locrmaps_for_woocommerce_styling = '';

		$options = get_option( Plugin::SLUG . '_options' );
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout' ] ) ) {
			$locrmaps_for_woocommerce_styling = trim( $options[ Plugin::SLUG . '_selection_layout' ] );
		}

		$stylings = array(
			''                    => __( 'Default', 'default' ),
			'wc-product-property' => __( 'WooCommerce Product Property', 'locrmaps-for-woocommerce-textdomain' ),
		);
		print '<select class="regular-text" name="' . esc_attr( Plugin::SLUG . '_options[' . Plugin::SLUG . '_selection_layout]' ) . '" id="' . esc_attr( Plugin::SLUG . '_selection_layout' ) . '">';
		foreach ( $stylings as $styling => $label ) {
			print '<option value="' . esc_attr( $styling ) . '" ' . selected( $locrmaps_for_woocommerce_styling, $styling, false ) . '>' . esc_html( $label ) . '</option>';
		}
		print '</select>';
	}

	/**
	 * This function renders the seletion layout label enabled field.
	 */
	public static function selection_layout_label_enabled_callback(): void {
		$selection_layout_enabled = '';

		$options = get_option( Plugin::SLUG . '_options' );
		if ( is_array( $options ) && isset( $options[ Plugin::SLUG . '_selection_layout_label_enabled' ] ) && is_string( $options[ Plugin::SLUG . '_selection_layout_label_enabled' ] ) ) {
			$selection_layout_enabled = checked( 'on', $options[ Plugin::SLUG . '_selection_layout_label_enabled' ], false );
		}

		print '<input type="checkbox" name="' . esc_attr( Plugin::SLUG . '_options[' . Plugin::SLUG . '_selection_layout_label_enabled]' ) . '" id="' . esc_attr( Plugin::SLUG . '_selection_layout_label_enabled' ) . '"' . esc_attr( $selection_layout_enabled ) . '/>';
	}

	/**
	 * This function renders the sizes field.
	 */
	public static function sizes_query_selector_callback(): void {
		$options = new Options( only_option: SizesQuerySelector::OPTION_KEY );

		$locrmaps_for_woocommerce_sizes_query_selector_placeholder = 'select#size';
		print '<input type="text" class="regular-text" name="locrmaps_for_woocommerce_options[' . esc_attr( SizesQuerySelector::OPTION_KEY ) . ']" id="' . esc_attr( SizesQuerySelector::OPTION_KEY ) . '" placeholder="' . esc_attr( $locrmaps_for_woocommerce_sizes_query_selector_placeholder ) . '" value="' . esc_attr( (string) $options->sizes_query_selector ) . '"/>';
	}

	/**
	 * This function renders the sizes shortcode field.
	 */
	public static function shortcode_for_icons_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_icons]' ) . '">';
	}

	/**
	 * This function renders the overlay templates shortcode field.
	 */
	public static function shortcode_for_overlay_templates_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_overlay_templates]' ) . '">';
	}

	/**
	 * This function renders the overlay templates base colors shortcode field.
	 */
	public static function shortcode_for_overlay_templates_base_colors_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_overlay_templates_base_colors]' ) . '">';
	}

	/**
	 * This function renders the overlay templates text colors shortcode field.
	 */
	public static function shortcode_for_overlay_templates_text_colors_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_overlay_templates_text_colors]' ) . '">';
	}

	/**
	 * This function renders the styles shortcode field.
	 */
	public static function shortcode_for_styles_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_styles]' ) . '">';
	}

	/**
	 * This function renders the text overlay shortcode field.
	 */
	public static function shortcode_for_text_overlay_callback(): void {
		print '<input type="text" class="regular-text" readonly="readonly" value="' . esc_attr( '[locrmaps_text_overlay]' ) . '">';
	}

	/**
	 * This function renders the submenu page.
	 */
	public static function render_submenu_page(): void {
		print '<div class="wrap">' . PHP_EOL;

		settings_errors();

		print '<h1>' . esc_html__( 'locrMAPS', 'locrmaps-for-woocommerce-textdomain' ) . ' &rsaquo; ' . esc_html__( 'WooCommerce', 'locrmaps-for-woocommerce-textdomain' ) . '</h1>' . PHP_EOL;
		print '<form id="locrmaps-for-woocommerce-settings-form" action="options.php" method="post">' . PHP_EOL;

		settings_fields( 'locrmaps_for_woocommerce_options_group' );
		do_settings_sections( Plugin::SLUG );
		submit_button();

		print '</form>' . PHP_EOL;
		print '</div>' . PHP_EOL;
	}
}
